import socket
import signal
import sys
import logging


from core.config import BankNodeConfig
from core.exceptions import ConfigError
from utils import setup_logger
from database.database_manager import DatabaseManager

class BankNode():
    def __init__(self):
        try:
            self.config = BankNodeConfig()

            setup_logger(self.config.verbosity)
            self.logger = logging.getLogger(__name__)
            
            self.logger.info("Config is valid")
            self.logger.debug("Starting Bank Node")

            self._setup_signals()

            self.database_manager = DatabaseManager()
        except ConfigError as e:
            print(e)
            self.exit_with_error()

    def _setup_signals(self):
        self.logger.debug("Setting up exit signal hooks")

        # Handle C standard signals
        signal.signal(signal.SIGTERM, self.gracefully_exit)
        signal.signal(signal.SIGINT, self.gracefully_exit)

        # Handle windows related signals
        if sys.platform == "win32":
            signal.signal(signal.CTRL_C_EVENT, self.gracefully_exit)
            signal.signal(signal.CTRL_BREAK_EVENT, self.gracefully_exit)
            signal.signal(signal.SIGBREAK, self.gracefully_exit)

    def start_server(self):
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as socket_server:
            socket_server.bind(("127.0.0.1", self.config.port))
            socket_server.listen()
            self.logger.info("Listening on %s:%s", "127.0.0.1", self.config.port)
            while True:
                conn, addr = socket_server.accept()
                with conn:
                    print(f"{addr} connected")
                    request = conn.recv(1024).decode("utf-8")
                    print(f"Request:\n{request}")

    def exit_with_error(self):
        self.cleanup()
        sys.exit(1)

    def cleanup(self):
        pass

    def gracefully_exit(self, signum, frame):
        signal_name = signal.Signals(signum).name
        self.logger.warning("Caught %s. Cleaning up before exiting", signal_name)
        self.cleanup()
        sys.exit(0)
